/*
  RCTractor.h - Library for controlling RC Tractors.
  Created by Oisin O'Conchubhair, September 20, 2015.
  Released into the public domain.

  This library uses parts from pcf8574.cpp written by Rob Tillaart.
*/

#include "Arduino.h"
#include "RCTGV1.h"
#include <RCTractorGuyI2C.h>
#include "RCTractorGuy.h"
#include <Servo.h>


/************************************************************/
/*       Functions used to control RC vehicles              */
/************************************************************/


RCTGV1::RCTGV1(){}

void RCTGV1::Define_Pins(int Steering, int Rear_Link, int Front_Link, int Left_Ind,int Right_Ind,int Head_Lights,int Tail_Lights,int Work_Lights,int Accessory_Lights,int Left_Beacon, int Right_Beacon, int Reverse)
{
	Tractor.Define_Pins(5, 94, 95, 6, 90, 97, Steering, Rear_Link, Front_Link, Left_Ind, Right_Ind, Head_Lights, Tail_Lights, Work_Lights, Accessory_Lights, Left_Beacon, Right_Beacon, Reverse);
	Tractor.Toggle_PCF_On(96);
}

void RCTGV1::Reverse_Motor_1(){Tractor.Reverse_Motor_1();}
void RCTGV1::Reverse_Motor_2(){Tractor.Reverse_Motor_2();}
void RCTGV1::Swap_Motor_1_2(){Tractor.Swap_Motor_1_2();}
void RCTGV1::Reverse_Motor_3(){Tractor.Reverse_Motor_3();}
void RCTGV1::Reverse_Motor_4(){Tractor.Reverse_Motor_4();}

void RCTGV1::begin(){
	Tractor.begin();
}

void RCTGV1::Trailer(uint8_t buf[16]){
	Tractor.Trailer(buf);
}

void RCTGV1::SetServoLimits(int Ser1_Min, int Ser1_Centre, int Ser1_Max, int Ser2_Min, int Ser2_Centre, int Ser2_Max, int Ser3_Min, int Ser3_Centre, int Ser3_Max){
	Tractor.SetServoLimits(Ser1_Min, Ser1_Centre, Ser1_Max, Ser2_Min, Ser2_Centre, Ser2_Max, Ser3_Min, Ser3_Centre, Ser3_Max);
}

void RCTGV1::MotorConfiguration(int MotorConfig){
	Tractor.MotorConfiguration(MotorConfig);
}

void RCTGV1::SteeringConfiguration(int SteeringConfig){
	Tractor.SteeringConfiguration(SteeringConfig);
}

void RCTGV1::LiftConfiguration(int Lift1Config, int Lift2Config){
	Tractor.LiftConfiguration(Lift1Config, Lift2Config);
}

void RCTGV1::Drive(int New_Drive_Val,int New_Steering_Val){
	Tractor.Drive(New_Drive_Val,New_Steering_Val);
}

void RCTGV1::LiftingArms(int New_Rear_Val,int New_Front_Val){
	Tractor.LiftingArms(New_Rear_Val, New_Front_Val);
}

void RCTGV1::Loader(int Raise_Val, int Raise_Sense, int Raise_Min, int Raise_Max, int Bucket_Val, int Bucket_Sense, int Bucket_Min, int Bucket_Max){
    Tractor.Loader(Raise_Val, Raise_Sense, Raise_Min, Raise_Max, Bucket_Val, Bucket_Sense, Bucket_Min, Bucket_Max);
}

void RCTGV1::Motor_3(int Third_Motor_Val){
	Tractor.Motor_3(Third_Motor_Val);
}

void RCTGV1::Motor_4(int Fourth_Motor_Val){
	Tractor.Motor_4(Fourth_Motor_Val);
}

void RCTGV1::Toggle_PCF_On(int _PCF_Pin){
	Tractor.Toggle_PCF_On(_PCF_Pin);
}

void RCTGV1::Toggle_PCF_Off(int _PCF_Pin){
	Tractor.Toggle_PCF_Off(_PCF_Pin);
}


void RCTGV1::Lights(int Light_Val, int Siku){
	Tractor.Lights(Light_Val, Siku);
}

void RCTGV1::Timer(){
	Tractor.Timer();
}

long RCTGV1::ReadVcc(){
	long result = Tractor.ReadVcc();
	return result;
}