/*
  RCTractor.h - Library for controlling RC Tractors.
  Created by Oisin O'Conchubhair, September 20, 2015.
  Released into the public domain.

  This library uses parts from pcf8574.cpp written by Rob Tillaart.
*/

#include <Arduino.h>
#include <RCTractorGuy.h>
#include <RCTractorGuyI2C.h>

#include <Servo.h>

// Pin Assignments
static int _Steering_Servo_Pin = 0;
static int _Rear_Link_Servo_Pin = 0;
static int _Front_Link_Servo_Pin = 0;
static int _Soft_PWM_Pins = 0;
static int _Motor_1_PWM_Pin = 0;
static int _Motor_1_Direction_1_Pin = 0;
static int _Motor_1_Direction_2_Pin = 0;
static int _Motor_2_PWM_Pin = 0;
static int _Motor_2_Direction_1_Pin = 0;
static int _Motor_2_Direction_2_Pin = 0;
static int _Motor_3_PWM_Pin = 0;
static int _Motor_3_Direction_1_Pin = 0;
static int _Motor_3_Direction_2_Pin = 0;
static int _Motor_4_PWM_Pin = 0;
static int _Motor_4_Direction_1_Pin = 0;
static int _Motor_4_Direction_2_Pin = 0;
static int _Head_Lights_Pin = 0;
static int _Tail_Lights_Pin = 0;
static int _Work_Lights_Pin = 0;
static int _Accessory_Lights_Pin = 0;
static int _Left_Indicator_Pin = 0;
static int _Right_Indicator_Pin = 0;
static int _Left_Beacon_Pin = 0;
static int _Right_Beacon_Pin = 0;
static int _Reverse_Pin = 0;
static int _Lift1_Config = 0;
static int _Lift2_Config = 0;
static int PWM_PINS[] = {3,5,6,9,10,11};
static int _PWM_Timer = 0;
static int _PWM_Timer_Max = 5;
static int _Tails_On = 0;

#define SOP '<'
#define EOP '>'

static int _Diff_Adj_1;
static int _Diff_Adj_2;
static int _Motor_1_PWM_Value;
static int _Motor_2_PWM_Value;
static int _Motor_3_PWM_Value;
static int _Motor_4_PWM_Value;
static int _Heads_PWM_Value;
static int _Tails_PWM_Value;
static int _Light_Val;
static int _Siku;
static int _Old_Light_Val;
unsigned long _Flash_Timer;
static int _Flash_Timer2;
static int _Left_Ind_Status;
static int _Right_Ind_Status;
static int _Beacon_Option = 2;
static int _Drive_Val = 0;

static int _PCF1_Data;
static int _PCF1_Data_Old;
static int _PCF2_Data;
static int _PCF2_Data_Old;
static int _PCF1_ID = 0x38;
static int _PCF2_ID = 0x39;

// Motor Configuration Variables

static int _Motor_Config = 1;
static int _Steering_Config = 1;


// Servo Variables
static int _Steering_Servo_Min = 0;
static int _Steering_Servo_Centre = 90;
static int _Steering_Servo_Max = 180;
static int _Rear_Link_Servo_Min = 0;
static int _Rear_Link_Servo_Centre = 90;
static int _Rear_Link_Servo_Max = 180;
static int _Front_Link_Servo_Min = 0;
static int _Front_Link_Servo_Centre = 90;
static int _Front_Link_Servo_Max = 180;
static int _Steering_Val = 0;
static int _Rear_Link_Val = 0;
static int _Front_Link_Val = 0;


unsigned long startMicros;
unsigned long currentMicros;
unsigned long timePassed;
unsigned long timePeriod = 5000;

/************************************************************/
/*       Functions used to control RC vehicles              */
/************************************************************/


RCTractor::RCTractor(){startMicros = micros();}

void RCTractor::Define_Pins(int Motor_1_PWM, int Motor_1_Dir_1, int Motor_1_Dir_2, int Motor_2_PWM, int Motor_2_Dir_1, int Motor_2_Dir_2, int Steering, int Rear_Link, int Front_Link, int Left_Ind,int Right_Ind,int Head_Lights,int Tail_Lights,int Work_Lights,int Accessory_Lights,int Left_Beacon, int Right_Beacon, int Reverse)
{

	_Motor_1_PWM_Pin = Motor_1_PWM;
	_Motor_1_Direction_1_Pin = Motor_1_Dir_1;
	_Motor_1_Direction_2_Pin = Motor_1_Dir_2;
	_Motor_2_PWM_Pin = Motor_2_PWM;
	_Motor_2_Direction_1_Pin = Motor_2_Dir_1;
	_Motor_2_Direction_2_Pin = Motor_2_Dir_2;
	_Steering_Servo_Pin = Steering;
	_Rear_Link_Servo_Pin = Rear_Link;
	_Front_Link_Servo_Pin = Front_Link;
	_Left_Indicator_Pin = Left_Ind;
	_Right_Indicator_Pin = Right_Ind;
	_Head_Lights_Pin = Head_Lights;
	_Tail_Lights_Pin = Tail_Lights;
	_Work_Lights_Pin = Work_Lights;
	_Accessory_Lights_Pin = Accessory_Lights;
	_Left_Beacon_Pin = Left_Beacon;
	_Right_Beacon_Pin = Right_Beacon;
	_Reverse_Pin = Reverse;

	if(_Motor_1_PWM_Pin!=0 && _Motor_1_PWM_Pin<89){pinMode(_Motor_1_PWM_Pin, OUTPUT);if(Check_PWM(_Motor_1_PWM_Pin)==0){bitSet(_Soft_PWM_Pins,0);}}
	if(_Motor_1_Direction_1_Pin!=0 && _Motor_1_Direction_1_Pin<89){pinMode(_Motor_1_Direction_1_Pin, OUTPUT);}
	if(_Motor_1_Direction_2_Pin!=0 && _Motor_1_Direction_2_Pin<89){pinMode(_Motor_1_Direction_2_Pin, OUTPUT);}
	if(_Motor_2_PWM_Pin!=0 && _Motor_2_PWM_Pin<89){pinMode(_Motor_2_PWM_Pin, OUTPUT);if(Check_PWM(_Motor_2_PWM_Pin)==0){bitSet(_Soft_PWM_Pins,1);}}
	if(_Motor_2_Direction_1_Pin!=0 && _Motor_2_Direction_1_Pin<89){pinMode(_Motor_2_Direction_1_Pin, OUTPUT);}
	if(_Motor_2_Direction_2_Pin!=0 && _Motor_2_Direction_2_Pin<89){pinMode(_Motor_2_Direction_2_Pin, OUTPUT);}
	if(_Motor_3_PWM_Pin!=0 && _Motor_3_PWM_Pin<89){pinMode(_Motor_3_PWM_Pin, OUTPUT);if(Check_PWM(_Motor_3_PWM_Pin)==0){bitSet(_Soft_PWM_Pins,2);}}
	if(_Motor_3_Direction_1_Pin!=0 && _Motor_3_Direction_1_Pin<89){pinMode(_Motor_3_Direction_1_Pin, OUTPUT);}
	if(_Motor_3_Direction_2_Pin!=0 && _Motor_3_Direction_2_Pin<89){pinMode(_Motor_3_Direction_2_Pin, OUTPUT);}
	if(_Motor_4_PWM_Pin!=0 && _Motor_4_PWM_Pin<89){pinMode(_Motor_4_PWM_Pin, OUTPUT);if(Check_PWM(_Motor_4_PWM_Pin)==0){bitSet(_Soft_PWM_Pins,3);}}
	if(_Motor_4_Direction_1_Pin!=0 && _Motor_4_Direction_1_Pin<89){pinMode(_Motor_4_Direction_1_Pin, OUTPUT);}
	if(_Motor_4_Direction_2_Pin!=0 && _Motor_4_Direction_2_Pin<89){pinMode(_Motor_4_Direction_2_Pin, OUTPUT);}
	if(_Steering_Servo_Pin!=0){pinMode(_Steering_Servo_Pin, OUTPUT);}
	if(_Rear_Link_Servo_Pin!=0){pinMode(_Rear_Link_Servo_Pin, OUTPUT);}
	if(_Front_Link_Servo_Pin!=0){pinMode(_Front_Link_Servo_Pin, OUTPUT);}
	if(_Left_Indicator_Pin!=0 && _Left_Indicator_Pin<89){pinMode(_Left_Indicator_Pin, OUTPUT);}
	if(_Right_Indicator_Pin!=0 && _Right_Indicator_Pin<89){pinMode(_Right_Indicator_Pin, OUTPUT);}
	if(_Head_Lights_Pin!=0 && _Head_Lights_Pin<89){pinMode(_Head_Lights_Pin, OUTPUT);if(Check_PWM(_Head_Lights_Pin)==0){bitSet(_Soft_PWM_Pins,6);}}
	if(_Tail_Lights_Pin!=0 && _Tail_Lights_Pin<89){pinMode(_Tail_Lights_Pin, OUTPUT);if(Check_PWM(_Tail_Lights_Pin)==0){bitSet(_Soft_PWM_Pins,7);}}
	if(_Work_Lights_Pin!=0 && _Work_Lights_Pin<89){pinMode(_Work_Lights_Pin, OUTPUT);}
	if(_Accessory_Lights_Pin!=0 && _Accessory_Lights_Pin<89){pinMode(_Accessory_Lights_Pin, OUTPUT);}
	if(_Left_Beacon_Pin!=0 && _Left_Beacon_Pin<89){pinMode(_Left_Beacon_Pin, OUTPUT);}
	if(_Right_Beacon_Pin!=0 && _Right_Beacon_Pin<89){pinMode(_Right_Beacon_Pin, OUTPUT);}
	if(_Reverse_Pin!=0 && _Reverse_Pin<89){pinMode(_Reverse_Pin, OUTPUT);}

	_PCF1_Data = B00000000;
	_PCF2_Data = B00000000;

	I2c.begin();
	I2c.write(_PCF1_ID,0x02,_PCF1_Data);
	I2c.write(_PCF2_ID,0x02,_PCF2_Data);
}


void RCTractor::Define_Pins_Single(int Motor_PWM_1, int Motor_PWM_2, int Steering, int Rear_Link, int Front_Link, int Left_Ind,int Right_Ind,int Head_Lights,int Tail_Lights,int Work_Lights,int Accessory_Lights,int Left_Beacon, int Right_Beacon, int Reverse)
{

	_Motor_1_PWM_Pin = Motor_PWM_1;
	_Motor_2_PWM_Pin = Motor_PWM_2;
	_Steering_Servo_Pin = Steering;
	_Rear_Link_Servo_Pin = Rear_Link;
	_Front_Link_Servo_Pin = Front_Link;
	_Left_Indicator_Pin = Left_Ind;
	_Right_Indicator_Pin = Right_Ind;
	_Head_Lights_Pin = Head_Lights;
	_Tail_Lights_Pin = Tail_Lights;
	_Work_Lights_Pin = Work_Lights;
	_Accessory_Lights_Pin = Accessory_Lights;
	_Left_Beacon_Pin = Left_Beacon;
	_Right_Beacon_Pin = Right_Beacon;
	_Reverse_Pin = Reverse;

	if(_Motor_1_PWM_Pin!=0){pinMode(_Motor_1_PWM_Pin, OUTPUT);}
	if(_Motor_2_PWM_Pin!=0){pinMode(_Motor_2_PWM_Pin, OUTPUT);}
	if(_Steering_Servo_Pin!=0){pinMode(_Steering_Servo_Pin, OUTPUT);}
	if(_Rear_Link_Servo_Pin!=0){pinMode(_Rear_Link_Servo_Pin, OUTPUT);}
	if(_Front_Link_Servo_Pin!=0){pinMode(_Front_Link_Servo_Pin, OUTPUT);}
	if(_Left_Indicator_Pin!=0 && _Left_Indicator_Pin<89){pinMode(_Left_Indicator_Pin, OUTPUT);}
	if(_Right_Indicator_Pin!=0 && _Right_Indicator_Pin<89){pinMode(_Right_Indicator_Pin, OUTPUT);}
	if(_Head_Lights_Pin!=0 && _Head_Lights_Pin<89){pinMode(_Head_Lights_Pin, OUTPUT);if(Check_PWM(_Head_Lights_Pin)==0){bitSet(_Soft_PWM_Pins,6);}}
	if(_Tail_Lights_Pin!=0 && _Tail_Lights_Pin<89){pinMode(_Tail_Lights_Pin, OUTPUT);if(Check_PWM(_Tail_Lights_Pin)==0){bitSet(_Soft_PWM_Pins,7);}}
	if(_Work_Lights_Pin!=0 && _Work_Lights_Pin<89){pinMode(_Work_Lights_Pin, OUTPUT);}
	if(_Accessory_Lights_Pin!=0 && _Accessory_Lights_Pin<89){pinMode(_Accessory_Lights_Pin, OUTPUT);}
	if(_Left_Beacon_Pin!=0 && _Left_Beacon_Pin<89){pinMode(_Left_Beacon_Pin, OUTPUT);}
	if(_Right_Beacon_Pin!=0 && _Right_Beacon_Pin<89){pinMode(_Right_Beacon_Pin, OUTPUT);}
	if(_Reverse_Pin!=0 && _Reverse_Pin<89){pinMode(_Reverse_Pin, OUTPUT);}

}


void RCTractor::Reverse_Motor_1()
{
	int Temp1 = _Motor_1_Direction_1_Pin;
	int Temp2 = _Motor_1_Direction_2_Pin;
	_Motor_1_Direction_1_Pin = Temp2;
	_Motor_1_Direction_2_Pin = Temp1;
}

void RCTractor::Reverse_Motor_2()
{
	int Temp1 = _Motor_2_Direction_1_Pin;
	int Temp2 = _Motor_2_Direction_2_Pin;
	_Motor_2_Direction_1_Pin = Temp2;
	_Motor_2_Direction_2_Pin = Temp1;
}

void RCTractor::Swap_Motor_1_2()
{
	int Temp1 = _Motor_1_Direction_1_Pin;
	int Temp2 = _Motor_1_Direction_2_Pin;
	int Temp3 = _Motor_1_PWM_Pin;
	int Temp4 = _Motor_2_Direction_1_Pin;
	int Temp5 = _Motor_2_Direction_2_Pin;
	int Temp6 = _Motor_2_PWM_Pin;

	_Motor_1_Direction_1_Pin = Temp4;
	_Motor_1_Direction_2_Pin = Temp5;
	_Motor_1_PWM_Pin = Temp6;
	_Motor_2_Direction_1_Pin = Temp1;
	_Motor_2_Direction_2_Pin = Temp2;
	_Motor_2_PWM_Pin = Temp3;
}

void RCTractor::Reverse_Motor_3()
{
	int Temp1 = _Motor_3_Direction_1_Pin;
	int Temp2 = _Motor_3_Direction_2_Pin;
	_Motor_3_Direction_1_Pin = Temp2;
	_Motor_3_Direction_2_Pin = Temp1;
}

void RCTractor::Reverse_Motor_4()
{
	int Temp1 = _Motor_4_Direction_1_Pin;
	int Temp2 = _Motor_4_Direction_2_Pin;
	_Motor_4_Direction_1_Pin = Temp2;
	_Motor_4_Direction_2_Pin = Temp1;
}

bool RCTractor::Check_PWM(int _PWM_PIN)
{
int arraySize = sizeof (PWM_PINS) / sizeof(int);

for( int i=0; i< arraySize; i++)
  if ( PWM_PINS[i] == _PWM_PIN)
    {return 1;
    }

return 0;
}

void RCTractor::Define_Loader_Pins(int Motor_3_PWM, int Motor_3_Dir_1, int Motor_3_Dir_2, int Motor_4_PWM, int Motor_4_Dir_1, int Motor_4_Dir_2)
{

	_Motor_3_PWM_Pin = Motor_3_PWM;
	_Motor_3_Direction_1_Pin = Motor_3_Dir_1;
	_Motor_3_Direction_2_Pin = Motor_3_Dir_2;
	_Motor_4_PWM_Pin = Motor_4_PWM;
	_Motor_4_Direction_1_Pin = Motor_4_Dir_1;
	_Motor_4_Direction_2_Pin = Motor_4_Dir_2;

	if(_Motor_3_PWM_Pin!=0 && _Motor_3_PWM_Pin<89){pinMode(_Motor_3_PWM_Pin, OUTPUT);}
	if(_Motor_3_Direction_1_Pin!=0 && _Motor_3_Direction_1_Pin<89){pinMode(_Motor_3_Direction_1_Pin, OUTPUT);}
	if(_Motor_3_Direction_2_Pin!=0 && _Motor_3_Direction_2_Pin<89){pinMode(_Motor_3_Direction_2_Pin, OUTPUT);}
	if(_Motor_4_PWM_Pin!=0 && _Motor_4_PWM_Pin<89){pinMode(_Motor_4_PWM_Pin, OUTPUT);}
	if(_Motor_4_Direction_1_Pin!=0 && _Motor_4_Direction_1_Pin<89){pinMode(_Motor_4_Direction_1_Pin, OUTPUT);}
	if(_Motor_4_Direction_2_Pin!=0 && _Motor_4_Direction_2_Pin<89){pinMode(_Motor_4_Direction_2_Pin, OUTPUT);}
}

void RCTractor::begin(){
	if(_Steering_Servo_Pin!=0){
		_Steering_Servo.attach(_Steering_Servo_Pin);
	}

	if(_Rear_Link_Servo_Pin!=0){
		_Rear_Link_Servo.attach(_Rear_Link_Servo_Pin);
	}

	if(_Front_Link_Servo_Pin!=0){
		_Front_Link_Servo.attach(_Front_Link_Servo_Pin);
	}

	  // Start Serial Communications
	  Serial.begin(9600);
}

void RCTractor::Trailer(uint8_t buf[16]){
	Serial.write(SOP);
	Serial.write(0x01);
	Serial.write(buf[1]);
	Serial.write(buf[2]);
	Serial.write(buf[3]);
	Serial.write(buf[4]);
	Serial.write(buf[5]);
	Serial.write(buf[6]);
	Serial.write(buf[7]);
	Serial.write(buf[8]);
	Serial.write(buf[9]);
	Serial.write(buf[10]);
	Serial.write(EOP);
}

void RCTractor::SetServoLimits(int Ser1_Min, int Ser1_Centre, int Ser1_Max, int Ser2_Min, int Ser2_Centre, int Ser2_Max, int Ser3_Min, int Ser3_Centre, int Ser3_Max){
_Steering_Servo_Min = Ser1_Min;
_Steering_Servo_Centre = Ser1_Centre;
_Steering_Servo_Max = Ser1_Max;
_Rear_Link_Servo_Min = Ser2_Min;
_Rear_Link_Servo_Centre = Ser2_Centre;
_Rear_Link_Servo_Max = Ser2_Max;
_Front_Link_Servo_Min = Ser3_Min;
_Front_Link_Servo_Centre = Ser3_Centre;
_Front_Link_Servo_Max = Ser3_Max;
}

void RCTractor::MotorConfiguration(int MotorConfig){
	_Motor_Config = MotorConfig;
}

void RCTractor::SteeringConfiguration(int SteeringConfig){
	_Steering_Config = SteeringConfig;
}

void RCTractor::LiftConfiguration(int Lift1Config, int Lift2Config){
	_Lift1_Config = Lift1Config;
	_Lift2_Config = Lift2Config;
}

void RCTractor::Toggle_PCF_On(int _PCF_Pin){
	switch (_PCF_Pin) {
		case 90:
			bitSet(_PCF1_Data,0);
		break;
		case 91:
			bitSet(_PCF1_Data,1);
		break;
		case 92:
			bitSet(_PCF1_Data,2);
		break;
		case 93:
			bitSet(_PCF1_Data,3);
		break;
		case 94:
			bitSet(_PCF1_Data,4);
		break;
		case 95:
			bitSet(_PCF1_Data,5);
		break;
		case 96:
			bitSet(_PCF1_Data,6);
		break;
		case 97:
			bitSet(_PCF1_Data,7);
		break;
		case 100:
			bitSet(_PCF2_Data,0);
		break;
		case 101:
			bitSet(_PCF2_Data,1);
		break;
		case 102:
			bitSet(_PCF2_Data,2);
		break;
		case 103:
			bitSet(_PCF2_Data,3);
		break;
		case 104:
			bitSet(_PCF2_Data,4);
		break;
		case 105:
			bitSet(_PCF2_Data,5);
		break;
		case 106:
			bitSet(_PCF2_Data,6);
		break;
		case 107:
			bitSet(_PCF2_Data,7);
		break;
	}
}

void RCTractor::Toggle_PCF_Off(int _PCF_Pin){
	switch (_PCF_Pin) {
		case 90:
			bitClear(_PCF1_Data,0);
		break;
		case 91:
			bitClear(_PCF1_Data,1);
		break;
		case 92:
			bitClear(_PCF1_Data,2);
		break;
		case 93:
			bitClear(_PCF1_Data,3);
		break;
		case 94:
			bitClear(_PCF1_Data,4);
		break;
		case 95:
			bitClear(_PCF1_Data,5);
		break;
		case 96:
			bitClear(_PCF1_Data,6);
		break;
		case 97:
			bitClear(_PCF1_Data,7);
		break;
		case 100:
			bitClear(_PCF2_Data,0);
		break;
		case 101:
			bitClear(_PCF2_Data,1);
		break;
		case 102:
			bitClear(_PCF2_Data,2);
		break;
		case 103:
			bitClear(_PCF2_Data,3);
		break;
		case 104:
			bitClear(_PCF2_Data,4);
		break;
		case 105:
			bitClear(_PCF2_Data,5);
		break;
		case 106:
			bitClear(_PCF2_Data,6);
		break;
		case 107:
			bitClear(_PCF2_Data,7);
		break;
	}
}

void RCTractor::Drive_Single(int New_Drive_Val,int New_Steering_Val){

	int _Steering_Val_Map = 90;

	_Steering_Val = New_Steering_Val;
	if (_Steering_Val < 123){
		_Steering_Val_Map = map(_Steering_Val, 0x00, 123, _Steering_Servo_Min, _Steering_Servo_Centre);
	}
	else if(_Steering_Val > 132){
		_Steering_Val_Map = map(_Steering_Val, 132, 0xFF, _Steering_Servo_Centre, _Steering_Servo_Max);
	}
	else{_Steering_Val_Map=_Steering_Servo_Centre;}

if(_Steering_Config == 1){
	_Steering_Servo.write(_Steering_Val_Map);
}
else if(_Steering_Config == 2){
								if (_Steering_Val_Map < (_Steering_Servo_Centre-2)){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
							    	_Motor_2_PWM_Value = map(_Steering_Val_Map, 85, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
									analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

									if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_2_Direction_1_Pin);}
									if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_2_Pin);}

								}
								else if (_Steering_Val_Map > (_Steering_Servo_Centre+2)){
							    	_Motor_2_PWM_Value = map(_Steering_Val_Map, 95, 180, 0,255);//drive_min, drive_max); // scale the data value for use with the motor

									analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor   // RCTractorDiff::write(0, 0);

									if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_1_Pin);}
									if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_2_Direction_2_Pin);}

								}
								else{
							    	_Motor_2_PWM_Value=0;
									analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

									if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_1_Pin);}
									if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_2_Pin);}
							}
}

	_Drive_Val = New_Drive_Val;
	Drive_Motor_Single();

}

void RCTractor::Drive_Motor_Single(){

	if (_Drive_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
		_Motor_1_PWM_Value = map(_Drive_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
		analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
		digitalWrite(_Motor_2_PWM_Pin,LOW);
	}
	else if (_Drive_Val > 135){
		_Motor_1_PWM_Value = map(_Drive_Val, 135, 255, 255,0);//drive_min, drive_max); // scale the data value for use with the motor
		analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
		digitalWrite(_Motor_2_PWM_Pin,HIGH);
	}
	else{
		analogWrite(_Motor_1_PWM_Pin,0);
		digitalWrite(_Motor_2_PWM_Pin,LOW);
	}

}

void RCTractor::Drive(int New_Drive_Val,int New_Steering_Val){
	int _Steering_Val_Map = 90;

	_Steering_Val = New_Steering_Val;
	if (_Steering_Val < 123){
		_Steering_Val_Map = map(_Steering_Val, 0x00, 123, _Steering_Servo_Min, _Steering_Servo_Centre);
	}
	else if(_Steering_Val > 132){
		_Steering_Val_Map = map(_Steering_Val, 132, 0xFF, _Steering_Servo_Centre, _Steering_Servo_Max);
	}
	else{_Steering_Val_Map=_Steering_Servo_Centre;}

if(_Steering_Config == 1){
	_Steering_Servo.write(_Steering_Val_Map);
}
else if(_Steering_Config == 2){
								if (_Steering_Val_Map < (_Steering_Servo_Centre-2)){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
							    	_Motor_2_PWM_Value = map(_Steering_Val_Map, 85, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
									analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

									if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_2_Direction_1_Pin);}
									if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_2_Pin);}

								}
								else if (_Steering_Val_Map > (_Steering_Servo_Centre+2)){
							    	_Motor_2_PWM_Value = map(_Steering_Val_Map, 95, 180, 0,255);//drive_min, drive_max); // scale the data value for use with the motor

									analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor   // RCTractorDiff::write(0, 0);

									if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_1_Pin);}
									if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_2_Direction_2_Pin);}

								}
								else{
							    	_Motor_2_PWM_Value=0;
									analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

									if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_1_Pin);}
									if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_2_Pin);}
							}
}

	_Drive_Val = New_Drive_Val;
	Drive_Motor();
}

void RCTractor::LiftingArms(int New_Rear_Val,int New_Front_Val){

	_Rear_Link_Val = map(New_Rear_Val, 0x00, 0xFF, _Rear_Link_Servo_Min, _Rear_Link_Servo_Max); // Scale data_val to between steering_min and steering_max
	_Rear_Link_Servo.write(_Rear_Link_Val);

	_Front_Link_Val = map(New_Front_Val, 0x00, 0xFF, _Front_Link_Servo_Min, _Front_Link_Servo_Max); // Scale data_val to between steering_min and steering_max
	_Front_Link_Servo.write(_Front_Link_Val);

}

void RCTractor::Motor_3(int _Third_Motor_Val){
	if (_Third_Motor_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
		_Motor_3_PWM_Value = map(_Third_Motor_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor

					if(bitRead(_Soft_PWM_Pins,2)==0){analogWrite(_Motor_3_PWM_Pin,_Motor_3_PWM_Value);}

					if(_Motor_3_Direction_1_Pin<89){digitalWrite(_Motor_3_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_3_Direction_1_Pin);}
					if(_Motor_3_Direction_2_Pin<89){digitalWrite(_Motor_3_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_2_Pin);}

	}
	else if (_Third_Motor_Val > 135){
		_Motor_3_PWM_Value = map(_Third_Motor_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor

					if(bitRead(_Soft_PWM_Pins,2)==0){analogWrite(_Motor_3_PWM_Pin,_Motor_3_PWM_Value);}


					if(_Motor_3_Direction_1_Pin<89){digitalWrite(_Motor_3_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_1_Pin);}
					if(_Motor_3_Direction_2_Pin<89){digitalWrite(_Motor_3_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_3_Direction_2_Pin);}

	}
	else{
		_Motor_3_PWM_Value=0;

					if(bitRead(_Soft_PWM_Pins,2)==0){analogWrite(_Motor_3_PWM_Pin,_Motor_3_PWM_Value);}

					if(_Motor_3_Direction_1_Pin<89){digitalWrite(_Motor_3_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_1_Pin);}
					if(_Motor_3_Direction_2_Pin<89){digitalWrite(_Motor_3_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_2_Pin);}

	}
}

void RCTractor::Motor_4(int _Fourth_Motor_Val){
	if (_Fourth_Motor_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
		_Motor_4_PWM_Value = map(_Fourth_Motor_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor

					if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

					if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_4_Direction_1_Pin);}
					if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_2_Pin);}

	}
	else if (_Fourth_Motor_Val > 135){
		_Motor_4_PWM_Value = map(_Fourth_Motor_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor

					if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

					if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_1_Pin);}
					if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_4_Direction_2_Pin);}

	}
	else{
		_Motor_4_PWM_Value=0;

					if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

					if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_1_Pin);}
					if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_2_Pin);}

	}
}

void RCTractor::Loader(int Raise_Val, int Raise_Sense, int Raise_Min, int Raise_Max, int Bucket_Val, int Bucket_Sense, int Bucket_Min, int Bucket_Max){

			if ((Raise_Val < 120) && (analogRead(Raise_Sense)>Raise_Min)){ // These if functions leave a buffer zone of 15 bits above and below the drive center value

					_Motor_3_PWM_Value = map(Raise_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
					_Motor_4_PWM_Value=(_Motor_3_PWM_Value/3);

					if(bitRead(_Soft_PWM_Pins,2)==0){analogWrite(_Motor_3_PWM_Pin,_Motor_3_PWM_Value);}
					if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

					if(_Motor_3_Direction_1_Pin<89){digitalWrite(_Motor_3_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_3_Direction_1_Pin);}
					if(_Motor_3_Direction_2_Pin<89){digitalWrite(_Motor_3_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_2_Pin);}
					if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_4_Direction_1_Pin);}
					if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_2_Pin);}

			}
			else if ((Raise_Val > 135) && (analogRead(Raise_Sense)<Raise_Max)){

					_Motor_3_PWM_Value = map(Raise_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
					_Motor_4_PWM_Value=(_Motor_3_PWM_Value/3);

					if(bitRead(_Soft_PWM_Pins,2)==0){analogWrite(_Motor_3_PWM_Pin,_Motor_3_PWM_Value);}
					if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

					if(_Motor_3_Direction_1_Pin<89){digitalWrite(_Motor_3_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_1_Pin);}
					if(_Motor_3_Direction_2_Pin<89){digitalWrite(_Motor_3_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_3_Direction_2_Pin);}
					if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_1_Pin);}
					if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_4_Direction_2_Pin);}
			}
			else{
				_Motor_3_PWM_Value=0;
				//_Motor_4_PWM_Value=0;

					if(bitRead(_Soft_PWM_Pins,2)==0){analogWrite(_Motor_3_PWM_Pin,_Motor_3_PWM_Value);}
				//	if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

					if(_Motor_3_Direction_1_Pin<89){digitalWrite(_Motor_3_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_1_Pin);}
					if(_Motor_3_Direction_2_Pin<89){digitalWrite(_Motor_3_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_2_Pin);}
				//	if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_1_Pin);}
				//	if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_2_Pin);}
			}


			if ((Bucket_Val < 120)&& (analogRead(Bucket_Sense)>Bucket_Min)){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
				_Motor_4_PWM_Value = map(Bucket_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor

					if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

					if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_4_Direction_1_Pin);}
					if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_2_Pin);}

			}
			else if ((Bucket_Val > 135) && (analogRead(Bucket_Sense)<Bucket_Max)){
				_Motor_4_PWM_Value = map(Bucket_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor

					if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

					if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_1_Pin);}
					if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_4_Direction_2_Pin);}

			}
			else{
				//_Motor_3_PWM_Value=0;
				_Motor_4_PWM_Value=0;

				//	if(bitRead(_Soft_PWM_Pins,2)==0){analogWrite(_Motor_3_PWM_Pin,_Motor_3_PWM_Value);}
					if(bitRead(_Soft_PWM_Pins,3)==0){analogWrite(_Motor_4_PWM_Pin,_Motor_4_PWM_Value);}

				//	if(_Motor_3_Direction_1_Pin<89){digitalWrite(_Motor_3_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_1_Pin);}
				//	if(_Motor_3_Direction_2_Pin<89){digitalWrite(_Motor_3_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_3_Direction_2_Pin);}
					if(_Motor_4_Direction_1_Pin<89){digitalWrite(_Motor_4_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_1_Pin);}
					if(_Motor_4_Direction_2_Pin<89){digitalWrite(_Motor_4_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_4_Direction_2_Pin);}
			}

}

void RCTractor::Drive_Motor(){
	if(_Motor_Config==1){
				if (_Drive_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
		    		_Motor_1_PWM_Value = map(_Drive_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
					analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
					if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_1_Direction_1_Pin);}
					if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_2_Pin);}
				}
   				else if (_Drive_Val > 135){
    				_Motor_1_PWM_Value = map(_Drive_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
					analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
					if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_1_Pin);}
					if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_1_Direction_2_Pin);}
 				}
   				else{
   	 				_Motor_1_PWM_Value=0;
    				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
					if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_1_Pin);}
					if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_2_Pin);}
   				}
	}

	if(_Motor_Config==2){
			if(_Steering_Val<120 && _Motor_1_PWM_Pin!=0 && _Motor_2_PWM_Pin!=0){
				_Diff_Adj_1 = map(_Steering_Val, 120, 0, 255,120);
				_Diff_Adj_2=255;
			}
			else if(_Steering_Val>135 && _Motor_1_PWM_Pin!=0 && _Motor_2_PWM_Pin!=0){
			    _Diff_Adj_2 = map(_Steering_Val, 135, 255, 255,120);//drive_min, drive_max); // scale the data value for use with the motor
			    _Diff_Adj_1=255;
			}
			else{
				_Diff_Adj_1=255;
				_Diff_Adj_2=255;
			}

			if (_Drive_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
		    	_Motor_1_PWM_Value = map(_Drive_Val, 120, 0, 0,_Diff_Adj_1);//drive_min, drive_max); // scale the data value for use with the motor
		    	_Motor_2_PWM_Value = map(_Drive_Val, 120, 0, 0,_Diff_Adj_2);//drive_min, drive_max); // scale the data value for use with the motor
				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

					if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_1_Direction_1_Pin);}
					if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_2_Pin);}
					if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_2_Direction_1_Pin);}
					if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_2_Pin);}

			}
			else if (_Drive_Val > 135){
		    	_Motor_1_PWM_Value = map(_Drive_Val, 135, 255, 0,_Diff_Adj_1);//drive_min, drive_max); // scale the data value for use with the motor
		    	_Motor_2_PWM_Value = map(_Drive_Val, 135, 255, 0,_Diff_Adj_2);//drive_min, drive_max); // scale the data value for use with the motor

				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor   // RCTractorDiff::write(0, 0);

					if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_1_Pin);}
					if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_1_Direction_2_Pin);}
					if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_1_Pin);}
					if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_2_Direction_2_Pin);}
			}
			else{
				_Motor_1_PWM_Value=0;
		    	_Motor_2_PWM_Value=0;
				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

					if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_1_Pin);}
					if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_2_Pin);}
					if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_1_Pin);}
					if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_2_Pin);}
			}
	}

	if(_Motor_Config==3){
				if(_Steering_Val<120 && _Motor_1_PWM_Pin!=0 && _Motor_2_PWM_Pin!=0){
								_Diff_Adj_1 = map(_Steering_Val, 120, 0, 255,0);
							}
							else if(_Steering_Val>135 && _Motor_1_PWM_Pin!=0 && _Motor_2_PWM_Pin!=0){
							    _Diff_Adj_2 = map(_Steering_Val, 135, 255, 255,0);//drive_min, drive_max); // scale the data value for use with the motor
							}
							else{
								_Diff_Adj_1=255;
								_Diff_Adj_2=255;
							}

							if (_Drive_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
						    	_Motor_1_PWM_Value = map(_Drive_Val, 120, 0, 0,_Diff_Adj_1);//drive_min, drive_max); // scale the data value for use with the motor
						    	_Motor_2_PWM_Value = map(_Drive_Val, 120, 0, 0,_Diff_Adj_2);//drive_min, drive_max); // scale the data value for use with the motor
								analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
								analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

								if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_1_Direction_1_Pin);}
								if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_2_Pin);}
								if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, HIGH);}else{Toggle_PCF_On(_Motor_2_Direction_1_Pin);}
								if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_2_Pin);}

							}
							else if (_Drive_Val > 135){
						    	_Motor_1_PWM_Value = map(_Drive_Val, 135, 255, 0,_Diff_Adj_1);//drive_min, drive_max); // scale the data value for use with the motor
						    	_Motor_2_PWM_Value = map(_Drive_Val, 135, 255, 0,_Diff_Adj_2);//drive_min, drive_max); // scale the data value for use with the motor

								analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
								analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor   // RCTractorDiff::write(0, 0);

								if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_1_Pin);}
								if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_1_Direction_2_Pin);}
								if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_1_Pin);}
								if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, HIGH);}else{Toggle_PCF_On(_Motor_2_Direction_2_Pin);}

							}
							else{
								_Motor_1_PWM_Value=0;
						    	_Motor_2_PWM_Value=0;
								analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
								analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

								if(_Motor_1_Direction_1_Pin<89){digitalWrite(_Motor_1_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_1_Pin);}
								if(_Motor_1_Direction_2_Pin<89){digitalWrite(_Motor_1_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_1_Direction_2_Pin);}
								if(_Motor_2_Direction_1_Pin<89){digitalWrite(_Motor_2_Direction_1_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_1_Pin);}
								if(_Motor_2_Direction_2_Pin<89){digitalWrite(_Motor_2_Direction_2_Pin, LOW);}else{Toggle_PCF_Off(_Motor_2_Direction_2_Pin);}
							}
	}
}

void RCTractor::Lights(int Light_Val, int Siku){
	_Light_Val = Light_Val;
	_Siku = Siku;
	if (_Drive_Val < 120){bitSet(_Light_Val,7);}

		if(_Old_Light_Val != _Light_Val){

			if((bitRead(_Light_Val,4)) == 1){ // Work Lights
			    if(_Head_Lights_Pin!=0){
					if(_Siku == 0){
						if(_Head_Lights_Pin<89){analogWrite(_Head_Lights_Pin, 255);}
						else{Toggle_PCF_On(_Head_Lights_Pin);}
					}
					else{
						if(_Head_Lights_Pin<89){analogWrite(_Head_Lights_Pin, 0);}
						else{Toggle_PCF_Off(_Head_Lights_Pin);}
					}
				} // Head Lights On
			    if(_Tail_Lights_Pin!=0){
					if(_Siku == 0){
							if(bitRead(_Soft_PWM_Pins,7)==0){
								if((120 < _Drive_Val) && (_Drive_Val < 135)){analogWrite(_Tail_Lights_Pin,255);}else{analogWrite(_Tail_Lights_Pin,125);}
							}
							else{
								_Tails_On = 1;
							}
					}
					else{
						if(_Tail_Lights_Pin<89){digitalWrite(_Tail_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Tail_Lights_Pin);}
					}
				} // Tail Lights On
			    if(_Work_Lights_Pin!=0){
					if(_Siku == 0){
						if(_Work_Lights_Pin<89){digitalWrite(_Work_Lights_Pin, HIGH);}
						else{Toggle_PCF_On(_Work_Lights_Pin);}
					}
					else{
						if(_Work_Lights_Pin<89){digitalWrite(_Work_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Work_Lights_Pin);}
					}
				} // Work Lights On
			    if(_Accessory_Lights_Pin!=0){
					if(_Siku == 0){
						if(_Accessory_Lights_Pin<89){digitalWrite(_Accessory_Lights_Pin, HIGH);}
						else{Toggle_PCF_On(_Accessory_Lights_Pin);}
					}
					else{
						if(_Accessory_Lights_Pin<89){digitalWrite(_Accessory_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Accessory_Lights_Pin);}
					}
				} // Accessory Lights On
			}
			else{
			    if(_Work_Lights_Pin!=0){
					if(_Siku == 1){
						if(_Work_Lights_Pin<89){digitalWrite(_Work_Lights_Pin, HIGH);}
						else{Toggle_PCF_On(_Work_Lights_Pin);}
					}
					else{
						if(_Work_Lights_Pin<89){digitalWrite(_Work_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Work_Lights_Pin);}
					}
				} // Work Lights Off
			}

			if((bitRead(_Light_Val,2)) == 1 && (bitRead(_Light_Val,3)) == 0){ // Dipped Head Lights
			    if(_Head_Lights_Pin!=0){
					if(_Siku == 0){
						if(_Head_Lights_Pin<89){analogWrite(_Head_Lights_Pin, 125);}
						else{Toggle_PCF_On(_Head_Lights_Pin);}
					}
					else{
						if(_Head_Lights_Pin<89){analogWrite(_Head_Lights_Pin, 125);}
						else{Toggle_PCF_Off(_Head_Lights_Pin);}
					}
				} // Head Lights 50%
			    if(_Tail_Lights_Pin!=0){
					if(_Siku == 0){
							if(bitRead(_Soft_PWM_Pins,7)==0){
								if((120 < _Drive_Val) && (_Drive_Val < 135)){analogWrite(_Tail_Lights_Pin,255);}else{analogWrite(_Tail_Lights_Pin,125);}
							}
							else{
								_Tails_On = 1;
							}
					}
					else{
						if(_Tail_Lights_Pin<89){digitalWrite(_Tail_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Tail_Lights_Pin);}
					}
				} // Tail Lights On
			    if(_Accessory_Lights_Pin!=0){
					if(_Siku == 0){
						if(_Accessory_Lights_Pin<89){digitalWrite(_Accessory_Lights_Pin, HIGH);}
						else{Toggle_PCF_On(_Accessory_Lights_Pin);}
					}
					else{
						if(_Accessory_Lights_Pin<89){digitalWrite(_Accessory_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Accessory_Lights_Pin);}
					}
				} // Accessory Lights On

			}

			if((bitRead(_Light_Val,2)) == 1 && (bitRead(_Light_Val,3)) == 1){  // Full Beams
			    if(_Head_Lights_Pin!=0){
					if(_Siku == 0){
						if(_Head_Lights_Pin<89){analogWrite(_Head_Lights_Pin, 255);}
						else{Toggle_PCF_On(_Head_Lights_Pin);}
					}
					else{
						if(_Head_Lights_Pin<89){analogWrite(_Head_Lights_Pin, 0);}
						else{Toggle_PCF_Off(_Head_Lights_Pin);}
					}
				} // Head Lights 100				if(_Tail_Lights_Pin!=0){
			    if(_Tail_Lights_Pin!=0){
					if(_Siku == 0){
							if(bitRead(_Soft_PWM_Pins,7)==0){
								if((120 < _Drive_Val) && (_Drive_Val < 135)){analogWrite(_Tail_Lights_Pin,255);}else{analogWrite(_Tail_Lights_Pin,125);}
							}
							else{
								_Tails_On = 1;
							}
					}
					else{
						if(_Tail_Lights_Pin<89){digitalWrite(_Tail_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Tail_Lights_Pin);}
					}
				} // Tail Lights On

			    if(_Accessory_Lights_Pin!=0){
					if(_Siku == 0){
						if(_Accessory_Lights_Pin<89){digitalWrite(_Accessory_Lights_Pin, HIGH);}
						else{Toggle_PCF_On(_Accessory_Lights_Pin);}
					}
					else{
						if(_Accessory_Lights_Pin<89){digitalWrite(_Accessory_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Accessory_Lights_Pin);}
					}
				} // Accessory Lights On
			}

			if((bitRead(_Light_Val,2)) == 0 && (bitRead(_Light_Val,3)) == 0){ // All Off
			    if(_Head_Lights_Pin!=0){
					if(_Siku == 1){
						if(_Head_Lights_Pin<89){analogWrite(_Head_Lights_Pin, 255);}
						else{Toggle_PCF_On(_Head_Lights_Pin);}
					}
					else{
						if(_Head_Lights_Pin<89){analogWrite(_Head_Lights_Pin, 0);}
						else{Toggle_PCF_Off(_Head_Lights_Pin);}
					}
				} // Head Lights 0%
			    if(_Tail_Lights_Pin!=0){
					if(_Siku == 1){
						if(_Tail_Lights_Pin<89){digitalWrite(_Tail_Lights_Pin, HIGH);}
						else{Toggle_PCF_On(_Tail_Lights_Pin);}
						_Tails_PWM_Value=255;
					}
					else{
						if(_Tail_Lights_Pin<89){digitalWrite(_Tail_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Tail_Lights_Pin);}
						_Tails_PWM_Value=0;
					}
				} // Tail Lights Off
				if(_Work_Lights_Pin!=0){
					_Tails_On = 0;
					if(_Siku == 1){
						if(_Work_Lights_Pin<89){digitalWrite(_Work_Lights_Pin, HIGH);}
						else{Toggle_PCF_On(_Work_Lights_Pin);}
					}
					else{
						if(_Work_Lights_Pin<89){digitalWrite(_Work_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Work_Lights_Pin);}
					}
				} // Work Lights Off
			    if(_Accessory_Lights_Pin!=0){
					if(_Siku == 1){
						if(_Accessory_Lights_Pin<89){digitalWrite(_Accessory_Lights_Pin, HIGH);}
						else{Toggle_PCF_On(_Accessory_Lights_Pin);}
					}
					else{
						if(_Accessory_Lights_Pin<89){digitalWrite(_Accessory_Lights_Pin, LOW);}
						else{Toggle_PCF_Off(_Accessory_Lights_Pin);}
					}
				} // Accessory Lights Off
			}

			if((bitRead(_Light_Val,7)) == 1){ // Work Lights
			    if(_Reverse_Pin!=0){
					if(_Siku == 0){
						if(_Reverse_Pin<89){digitalWrite(_Reverse_Pin, HIGH);}
						else{Toggle_PCF_On(_Reverse_Pin);}
					}
					else{
						if(_Reverse_Pin<89){digitalWrite(_Reverse_Pin, LOW);}
						else{Toggle_PCF_Off(_Reverse_Pin);}
					}
				} // Head Lights On
			}
			else{ // Reverse Lights
			    if(_Reverse_Pin!=0){
					if(_Siku == 0){
						if(_Reverse_Pin<89){digitalWrite(_Reverse_Pin, LOW);}
						else{Toggle_PCF_Off(_Reverse_Pin);}
					}
					else{
						if(_Reverse_Pin<89){digitalWrite(_Reverse_Pin, HIGH);}
						else{Toggle_PCF_On(_Reverse_Pin);}
					}
				} // Head Lights On
			}

			_Old_Light_Val = _Light_Val;
	}
}

void RCTractor::Timer(){
if(_PCF1_Data!=_PCF1_Data_Old){I2c.write(_PCF1_ID,0x02,_PCF1_Data);_PCF1_Data_Old=_PCF1_Data;}
if(_PCF2_Data!=_PCF2_Data_Old){I2c.write(_PCF2_ID,0x02,_PCF2_Data);_PCF2_Data_Old=_PCF2_Data;}


currentMicros = micros();
timePassed = currentMicros-startMicros;

//_Flash_Timer = _Flash_Timer+1;
	BeaconControl();



	if (_Flash_Timer == (50000/timePeriod) && _Flash_Timer2 == 10){ // If the desired on time is reached

		if((bitRead(_Light_Val,1)) == 1 && (bitRead(_Light_Val,0)) == 0){
	    	LeftIndicator();
	    }
	    else if((bitRead(_Light_Val,1)) == 0 && (bitRead(_Light_Val,0)) == 1){
	    	RightIndicator();
	    }
	    else if((bitRead(_Light_Val,1)) == 1 && (bitRead(_Light_Val,0)) == 1){
	    	HazardIndicator();
	    }
	    else{
	    	IndicatorsOff();
	    }
	    	_Flash_Timer = 0;
	    	_Flash_Timer2 = 0;
	}
	else if (_Flash_Timer == (50000/timePeriod)){ // If the desired on time is reached
	  	 // Clear ind_timer
	  	_Flash_Timer = 0;
	  	_Flash_Timer2 = _Flash_Timer2+1; // Incrememt ind_timer2
  	}


if (timePassed > timePeriod){_PWM_Timer = _PWM_Timer + 1;_Flash_Timer = _Flash_Timer+1;startMicros = micros();}



if(_PWM_Timer > _PWM_Timer_Max){_PWM_Timer=0;}

int New_Motor_3_PWM_Value = map(_Motor_3_PWM_Value, 0, 255, 0, (_PWM_Timer_Max));
int New_Motor_4_PWM_Value = map(_Motor_4_PWM_Value, 0, 255, 0, (_PWM_Timer_Max));
if(_PWM_Timer < New_Motor_3_PWM_Value){digitalWrite(_Motor_3_PWM_Pin,HIGH);}else{digitalWrite(_Motor_3_PWM_Pin,LOW);}
if(_PWM_Timer < New_Motor_4_PWM_Value){digitalWrite(_Motor_4_PWM_Pin,HIGH);}else{digitalWrite(_Motor_4_PWM_Pin,LOW);}

if(_Tails_On!=0){
if((120 < _Drive_Val) && (_Drive_Val < 135)){digitalWrite(_Tail_Lights_Pin,HIGH);}else{if (_PWM_Timer % 2 == 0){digitalWrite(_Tail_Lights_Pin,LOW);}else{digitalWrite(_Tail_Lights_Pin,HIGH);}}
}


/*	// Read 1.1V reference against AVcc
	ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
	delay(2); // Wait for Vref to settle
	ADCSRA |= _BV(ADSC); // Convert
	while (bit_is_set(ADCSRA,ADSC));
	_Vehicle_Voltage = ADCL;
	_Vehicle_Voltage |= ADCH<<8;
	_Vehicle_Voltage = (1126400L / _Vehicle_Voltage); // Back-calculate AVcc in mV


if(_Vehicle_Voltage<2900){
	digitalWrite(_RCTG0003_M1_PWM,LOW);
	digitalWrite(_RCTG0003_M2_PWM,LOW);
	digitalWrite(_RCTG0003_M3_PWM,LOW);
	digitalWrite(_RCTG0003_M4_PWM,LOW);
	delay(1000);
}*/




}

// Indicator control functions
void RCTractor::LeftIndicator(){
	if (_Left_Ind_Status == 0){ // If left indicator is off
    	if(_Left_Indicator_Pin!=0){
			if(_Siku == 0){
				if(_Left_Indicator_Pin<89){digitalWrite(_Left_Indicator_Pin, HIGH);}
				else{Toggle_PCF_On(_Left_Indicator_Pin);}
			}
			else{
				if(_Left_Indicator_Pin<89){digitalWrite(_Left_Indicator_Pin, LOW);}
				else{Toggle_PCF_Off(_Left_Indicator_Pin);}
			}

		} // turn it on
    	_Left_Ind_Status = 1;
    	Serial.write(SOP);
    	Serial.write(0x02);
    	Serial.write(0x01);
    	Serial.write(EOP);

  	}
  	else{ // If left indicator is on
    	IndicatorsOff();
  	}
}

void RCTractor::RightIndicator(){
  	if (_Right_Ind_Status == 0){ // If right indicator is off
    	if(_Right_Indicator_Pin!=0){
			if(_Siku == 0){
				if(_Right_Indicator_Pin<89){digitalWrite(_Right_Indicator_Pin, HIGH);}
				else{Toggle_PCF_On(_Right_Indicator_Pin);}
			}
			else{
				if(_Right_Indicator_Pin<89){digitalWrite(_Right_Indicator_Pin, LOW);}
				else{Toggle_PCF_Off(_Right_Indicator_Pin);}
			}

		} // turn it on
    	_Right_Ind_Status = 1;
    	Serial.write(SOP);
    	Serial.write(0x02);
    	Serial.write(0x02);
    	Serial.write(EOP);
  	}
  	else{ // If right indicator is on
    	IndicatorsOff();
  	}
}

void RCTractor::HazardIndicator(){
	if (_Left_Ind_Status == 0){ // If hazards are off
    	if(_Left_Indicator_Pin!=0){
			if(_Siku == 0){
				if(_Left_Indicator_Pin<89){digitalWrite(_Left_Indicator_Pin, HIGH);}
				else{Toggle_PCF_On(_Left_Indicator_Pin);}
			}
			else{
				if(_Left_Indicator_Pin<89){digitalWrite(_Left_Indicator_Pin, LOW);}
				else{Toggle_PCF_Off(_Left_Indicator_Pin);}
			}

		} // turn it on
    	if(_Right_Indicator_Pin!=0){
			if(_Siku == 0){
				if(_Right_Indicator_Pin<89){digitalWrite(_Right_Indicator_Pin, HIGH);}
				else{Toggle_PCF_On(_Right_Indicator_Pin);}
			}
			else{
				if(_Right_Indicator_Pin<89){digitalWrite(_Right_Indicator_Pin, LOW);}
				else{Toggle_PCF_Off(_Right_Indicator_Pin);}
			}

		} // turn it on
    	Serial.write(SOP);
    	Serial.write(0x02);
    	Serial.write(0x03);
    	Serial.write(EOP);
    	_Left_Ind_Status = 1;
  	}
  	else{ // If hazards are on
    	IndicatorsOff();
    }
}

void RCTractor::IndicatorsOff(){
    	if(_Left_Indicator_Pin!=0){
			if(_Siku == 1){
				if(_Left_Indicator_Pin<89){digitalWrite(_Left_Indicator_Pin, HIGH);}
				else{Toggle_PCF_On(_Left_Indicator_Pin);}
			}
			else{
				if(_Left_Indicator_Pin<89){digitalWrite(_Left_Indicator_Pin, LOW);}
				else{Toggle_PCF_Off(_Left_Indicator_Pin);}
			}

		} // turn it on
    	if(_Right_Indicator_Pin!=0){
			if(_Siku == 1){
				if(_Right_Indicator_Pin<89){digitalWrite(_Right_Indicator_Pin, HIGH);}
				else{Toggle_PCF_On(_Right_Indicator_Pin);}
			}
			else{
				if(_Right_Indicator_Pin<89){digitalWrite(_Right_Indicator_Pin, LOW);}
				else{Toggle_PCF_Off(_Right_Indicator_Pin);}
			}
		} // turn it on
    _Left_Ind_Status = 0;
    _Right_Ind_Status = 0;
	Serial.write(SOP);
	Serial.write(0x02);
	Serial.write(0x00);
	Serial.write(EOP);
}

void RCTractor::BeaconControl(){
if(_Beacon_Option==1){
    	if (_Flash_Timer2 == 0){
      		if(_Left_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Left_Beacon_Pin);}
				}
				else{
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Left_Beacon_Pin);}
				}
			}
      		if(_Right_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Right_Beacon_Pin);}
				}
				else{
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Right_Beacon_Pin);}
				}
			}
    	}
    	else if (_Flash_Timer2 == 5){
      		if(_Left_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Left_Beacon_Pin);}
				}
				else{
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Left_Beacon_Pin);}
				}
			}
      		if(_Right_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Right_Beacon_Pin);}
				}
				else{
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Right_Beacon_Pin);}
				}
			}
    	}
  	}
  	else if(_Beacon_Option==2){
    	if (_Flash_Timer2 == 0){
      		if(_Left_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Left_Beacon_Pin);}
				}
				else{
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Left_Beacon_Pin);}
				}
			}
    	}
    	if (_Flash_Timer2 == 1){
      		if(_Left_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Left_Beacon_Pin);}
				}
				else{
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Left_Beacon_Pin);}
				}
			}
    	}
    	if (_Flash_Timer2 == 2){
      		if(_Left_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Left_Beacon_Pin);}
				}
				else{
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Left_Beacon_Pin);}
				}
			}
    	}
    	if (_Flash_Timer2 == 3){
      		if(_Left_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Left_Beacon_Pin);}
				}
				else{
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Left_Beacon_Pin);}
				}
			}
    	}

    	if (_Flash_Timer2 == 6){
      		if(_Right_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Right_Beacon_Pin);}
				}
				else{
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Right_Beacon_Pin);}
				}
			}
    	}
    	if (_Flash_Timer2 == 7){
      		if(_Right_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Right_Beacon_Pin);}
				}
				else{
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Right_Beacon_Pin);}
				}
			}
    	}
    	if (_Flash_Timer2 == 8){
      		if(_Right_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Right_Beacon_Pin);}
				}
				else{
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Right_Beacon_Pin);}
				}
			}
    	}
    	if (_Flash_Timer2 == 9){
      		if(_Right_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Right_Beacon_Pin);}
				}
				else{
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Right_Beacon_Pin);}
				}
			}
    	}
 	}
  	else{
      		if(_Left_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Left_Beacon_Pin);}
				}
				else{
					if(_Left_Beacon_Pin<89){digitalWrite(_Left_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Left_Beacon_Pin);}
				}
			}
      		if(_Right_Beacon_Pin!=0){
				if(_Siku == 0){
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, LOW);}
					else{Toggle_PCF_Off(_Right_Beacon_Pin);}
				}
				else{
					if(_Right_Beacon_Pin<89){digitalWrite(_Right_Beacon_Pin, HIGH);}
					else{Toggle_PCF_On(_Right_Beacon_Pin);}
				}
			}
  	}

}

long RCTractor::ReadVcc(){
	long result;
	// Read 1.1V reference against AVcc
	ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
	delay(2); // Wait for Vref to settle
	ADCSRA |= _BV(ADSC); // Convert
	while (bit_is_set(ADCSRA,ADSC));
	result = ADCL;
	result |= ADCH<<8;
	result = (1126400L / result); // Back-calculate AVcc in mV
	return result;
}
