/*
  RCTractor.h - Library for controlling RC Tractors.
  Created by Oisin O'Conchubhair, September 20, 2015.
  Released into the public domain.
*/
#ifndef RCTractorGuy_h
#define RCTractorGuy_h

#if defined(ARDUINO) && ARDUINO >= 100
#include <Arduino.h>
#else
#include <WProgram.h>
#endif

#include <Servo.h>


class RCTractor
{
  public:
    RCTractor();
	void Define_Pins(int Motor_1_PWM, int Motor_1_Dir_1, int Motor_1_Dir_2, int Motor_2_PWM, int Motor_2_Dir_1, int Motor_2_Dir_2, int Steering, int Rear_Link, int Front_Link, int Left_Ind,int Right_Ind,int Head_Lights,int Tail_Lights,int Work_Lights,int Accessory_Lights,int Left_Beacon, int Right_Beacon, int Reverse);
	void Define_Pins_Single(int Motor_PWM_1, int Motor_PWM_2, int Steering, int Rear_Link, int Front_Link, int Left_Ind,int Right_Ind,int Head_Lights,int Tail_Lights,int Work_Lights,int Accessory_Lights,int Left_Beacon, int Right_Beacon, int Reverse);
	void Define_Loader_Pins(int Motor_3_PWM, int Motor_3_Dir_1, int Motor_3_Dir_2, int Motor_4_PWM, int Motor_4_Dir_1, int Motor_4_Dir_2);
    void Drive(int Drive_Val,int Steering_Val);
    void Drive_Single(int Drive_Val,int Steering_Val);
    void LiftingArms(int Rear_Val, int Front_Val);
    void Loader(int Raise_Val, int Raise_Sense, int Raise_Min, int Raise_Max, int Bucket_Val, int Bucket_Sense, int Bucket_Min, int Bucket_Max);
    void Motor_3(int Third_Motor_Val);
    void Motor_4(int Fourth_Motor_Val);
	void Reverse_Motor_1();
	void Reverse_Motor_2();
	void Swap_Motor_1_2();
	void Reverse_Motor_3();
	void Reverse_Motor_4();
    void Lights(int Light_Val, int Siku);
    void Timer();
    void begin();
    void Trailer(uint8_t buf[16]);
    void SetServoLimits(int Ser1_Min, int Ser1_Centre, int Ser1_Max, int Ser2_Min, int Ser2_Centre, int Ser2_Max, int Ser3_Min, int Ser3_Centre, int Ser3_Max);
    void MotorConfiguration(int MotorConfig);
    void SteeringConfiguration(int SteeringConfig);
    void LiftConfiguration(int Lift1Config, int Lift2Config);
	long ReadVcc();
	void Toggle_PCF_On(int PCF_Pin);
	void Toggle_PCF_Off(int PCF_Pin);

  private:
	Servo _Steering_Servo;
	Servo _Rear_Link_Servo;
	Servo _Front_Link_Servo;
	void Drive_Motor();
	void Drive_Motor_Single();
	void LeftIndicator();
	void RightIndicator();
	void HazardIndicator();
	void IndicatorsOff();
  	void BeaconControl();
  	bool Check_PWM(int _PWM_PIN);
};


#endif
