/*

*/

// This section describes the vehicle

// Variables used in the program
byte ID = 14; // Vehicle ID Number
int Active = 1; // Variable to determine if the tractor is in active
int Last_Data_Timer = 0;
int Controller_Locked = 0;
int Receive_Count = 0;

int Controller_ID = 0;

uint8_t buf[16]; // Array to store recieved data

#include <SPI.h>
#include "RCTractorGuyRF24.h"
#include "RCTractorGuynRF24L01.h"
#include "RCTGV1.h"
#include "RCTractorGuy.h"
#include <avr/wdt.h>

// Set up nRF24L01 radio on SPI bus plus pins 9 & 10
RF24 radio(9, 10);

// Initialise Vehicle
RCTGV1 P379;

// Controller Address
const uint64_t Controller[5] = { 0xE8E8F0F0E1LL, 0xE8E8F0F0E2LL, 0xE8E8F0F0E3LL, 0xE8E8F0F0E4LL, 0xE8E8F0F0E5LL};
const uint64_t Controller_Listen[5] = { 0x3A3A3A3AD1LL, 0x3A3A3A3AD2LL, 0x3A3A3A3AD3LL, 0x3A3A3A3AD4LL, 0x3A3A3A3AD5LL };

void setup(void)
{
  // Start the NRF24L01
  radio.begin();

  /*(Steering, Rear_Link, Front_Link, Left_Ind, Right_Ind, Head_Lights, Tail_Lights, Work_Lights,
  Accessory_Lights, Left_Beacon, Right_Beacon, Reverse_Light)*/
  P379.Define_Pins(8, 0, 0, A0, A2, 3, 2, 0, 0, 0, 0, A1);
  
  // Initialise the library
  P379.begin();

  // Set the servo limits
  // (Steering Servo Min, Steering Servo Max, Rear Link Servo Min, Rear Link Servo Max, Front Link Servo Min, Front Link Servo Max)
  P379.SetServoLimits(70, 92, 116, 0, 90, 180, 0, 90, 180);

  // Set the motor configuration
  // (Motor configuration 1 = 1 motor or 2 = 2 motors or 3 = tracks, Do you use a PCF8574 0=no or 1=yes)
  P379.MotorConfiguration(1);
 
// Set the steering configuration
// (Steering configuration 1 = servo or 2 = motor, Do you use a PCF8574 0=no or 1=yes)
 P379.SteeringConfiguration(1);
  
  radio.openReadingPipe(1, Controller[0]); // Set Address of Controller 1
  radio.openReadingPipe(2, Controller[1]); // Set Address of Controller 2
  radio.openReadingPipe(3, Controller[2]); // Set Address of Controller 1
  radio.openReadingPipe(4, Controller[3]); // Set Address of Controller 2
  radio.openReadingPipe(5, Controller[4]); // Set Address of Controller 2

  radio.startListening();

    Serial.begin(9600);
    wdt_enable(WDTO_250MS);
}

void loop(void)
{

    buf[0] = 0;

    if ( radio.available() )
    {
        // Fetch the payload.
       radio.read( buf, sizeof(buf));

      // We recieved a string of ten bytes, four ID, one command and six data values but they may not be for this tractor
      if (buf[0] == ID) { // Confirm that the correct vehicle ID has been recieved


      int pipe_num = buf[11];

      long Vehicle_Voltage = P379.ReadVcc();

     // First, stop listening so we can talk
      radio.stopListening();

      // Open the correct pipe for writing
      radio.openWritingPipe(Controller_Listen[pipe_num]);

      // Send battery voltage back.
      radio.write( &Vehicle_Voltage, sizeof(Vehicle_Voltage) );

      // Now, resume listening so we catch the next packets.
      radio.startListening();

        P379.Drive(buf[2], buf[3]);
        P379.Lights(buf[9],0);        // Update the lights
        P379.Trailer(buf);  // Send Command to any trailers
        
	if(bitRead(buf[9],4)==1){
	tone(A3, 55, 300);
	}

        Last_Data_Timer = 10000;
        Active = 1;
      }
    }
    else if (Last_Data_Timer != 0) {
      Last_Data_Timer = Last_Data_Timer - 1;
    }
    else if (Last_Data_Timer == 0 && Active == 1) {
      Active = 0;
      P379.Lights(0x03,0);
      P379.Drive(125, 125);
    }

    P379.Timer();
    wdt_reset();

    /* if (Active == 1) {
      }
      else if (Controller_Locked != 0) {
       All_Controllers();
       Controller_Locked = 0;
      }*/

}
