/*

*/

// This section describes the vehicle

// Variables used in the program
byte ID = 7; // Vehicle ID Number
int Active = 1; // Variable to determine if the tractor is in active
int Last_Data_Timer = 0;
int Controller_Locked = 0;
int Receive_Count = 0;

int Controller_ID = 0;

uint8_t buf[16]; // Array to store recieved data

#include <SPI.h>
#include "RCTractorGuyRF24.h"
#include "RCTractorGuynRF24L01.h"
#include "RCTGV3Q.h"
#include "RCTractorGuy.h"
#include <avr/wdt.h>

// Set up nRF24L01 radio on SPI bus plus pins 9 & 10
RF24 radio(9, 10);


RCTGV3Q Fendt;

// Controller Address
const uint64_t Controller[5] = { 0xE8E8F0F0E1LL, 0xE8E8F0F0E2LL, 0xE8E8F0F0E3LL, 0xE8E8F0F0E4LL, 0xE8E8F0F0E5LL};
const uint64_t Controller_Listen[5] = { 0x3A3A3A3AD1LL, 0x3A3A3A3AD2LL, 0x3A3A3A3AD3LL, 0x3A3A3A3AD4LL, 0x3A3A3A3AD5LL };

void setup(void)
{
  // Start the NRF24L01
  radio.begin();

/*(Steering, Rear_Link, Front_Link, Left_Ind, Right_Ind, Head_Lights, Tail_Lights, Work_Lights,
Accessory_Lights, Left_Beacon, Right_Beacon, Reverse Light)*/
  Fendt.Define_Pins(4, 2, 0, 100, 101, 3, 102, 90, 0, A1, A0, 0);
  Fendt.Define_Loader_Pins();
  
  // Initialise the library
  Fendt.begin();

  // Set the servo limits
  // (Steering Servo Min, Steering Servo Max, Rear Link Servo Min, Rear Link Servo Max, Front Link Servo Min, Front Link Servo Max)
  Fendt.SetServoLimits(0, 60, 120, 30, 90, 145,0, 90, 180);

  // Set the motor configuration
  // (Motor configuration 1 = 1 motor or 2 = 2 motors or 3 = tracks)
  Fendt.MotorConfiguration(1);

// Set the steering configuration
// (Steering configuration 1 = servo or 2 = motor)
 Fendt.SteeringConfiguration(1);

  radio.openReadingPipe(1, Controller[0]); // Set Address of Controller 1
  radio.openReadingPipe(2, Controller[1]); // Set Address of Controller 2
  radio.openReadingPipe(3, Controller[2]); // Set Address of Controller 1
  radio.openReadingPipe(4, Controller[3]); // Set Address of Controller 2
  radio.openReadingPipe(5, Controller[4]); // Set Address of Controller 2

  radio.startListening();

    Serial.begin(9600);
    wdt_enable(WDTO_250MS);    
}

void loop(void)
{

    buf[0] = 0;

    if ( radio.available() )
    {
        // Fetch the payload.
       radio.read( buf, sizeof(buf));

      // We recieved a string of ten bytes, four ID, one command and six data values but they may not be for this tractor
      if (buf[0] == ID) { // Confirm that the correct vehicle ID has been recieved

      int pipe_num = buf[11];

      long Vehicle_Voltage = Fendt.ReadVcc();

      Fendt.Timer();

     // First, stop listening so we can talk
      radio.stopListening();
      // Open the correct pipe for writing
      radio.openWritingPipe(Controller_Listen[pipe_num]);
      // Send battery voltage back.
      radio.write( &Vehicle_Voltage, sizeof(Vehicle_Voltage) );
      // Now, resume listening so we catch the next packets.
      radio.startListening();

      Fendt.Timer();

        int steer_inv = map(buf[3],0,255,255,0);
        Fendt.Drive(buf[2], steer_inv);
        Fendt.Timer();
        Fendt.LiftingArms(buf[5], buf[6]);
        Fendt.Timer();
        int loader_inv = map(buf[1],0,255,255,0);
        int loader_inv2 = map(buf[4],0,255,255,0);
        Fendt.Loader(loader_inv, A7, 480, 800, loader_inv2, A2, 500, 950);
        Fendt.Timer();
        Fendt.Lights(buf[9],1);        // Update the lights (Light Data, Invert Output)
        Fendt.Timer();            
        Fendt.Trailer(buf);	// Send Command to any trailers

        Last_Data_Timer = 10000;
        Active = 1;
      }
    }
    else if (Last_Data_Timer != 0) {
      Last_Data_Timer = Last_Data_Timer - 1;
    }
    else if (Last_Data_Timer == 0 && Active == 1) {
      Active = 0;
      Fendt.Lights(0x03,1);
      Fendt.Drive(125, 125);
    }

    Fendt.Timer();
    wdt_reset();    


    /* if (Active == 1) {
      }
      else if (Controller_Locked != 0) {
       All_Controllers();
       Controller_Locked = 0;
      }*/

}