/*

*/

// This section describes the vehicle

// Variables used in the program
byte ID = 1; // Vehicle ID Number
int Active = 1; // Variable to determine if the tractor is in active
int Last_Data_Timer = 0;
int Controller_Locked = 0;
int Receive_Count = 0;

int Controller_ID = 0;

uint8_t buf[16]; // Array to store recieved data

#include <SPI.h>
#include "RCTractorGuyRF24.h"
#include "RCTractorGuynRF24L01.h"
#include "RCTGV4_1D.h"
#include "RCTractorGuy.h"
#include <avr/wdt.h>

// Set up nRF24L01 radio on SPI bus plus pins 9 & 10
RF24 radio(9, 10);

// Initialise Tractor
RCTGV4_1D Monster_Crawler;

// Controller Address
const uint64_t Controller[5] = { 0xE8E8F0F0E1LL, 0xE8E8F0F0E2LL, 0xE8E8F0F0E3LL, 0xE8E8F0F0E4LL, 0xE8E8F0F0E5LL};
const uint64_t Controller_Listen[5] = { 0x3A3A3A3AD1LL, 0x3A3A3A3AD2LL, 0x3A3A3A3AD3LL, 0x3A3A3A3AD4LL, 0x3A3A3A3AD5LL };

void setup(void)
{
  // Start the NRF24L01
  radio.begin();

/*(Steering, Rear_Link, Front_Link, Left_Ind, Right_Ind, Head_Lights, Tail_Lights, Work_Lights,
Accessory_Lights, Left_Beacon, Right_Beacon, Reverse_Light)*/
  Monster_Crawler.Define_Pins(0, 0, 0, A2, A3, 3, 2, 7, 0, A0, A1, 4);
  
  // Initialise the library
  Monster_Crawler.begin();

  // Set the servo limits
  // (Steering Servo Min, Steering Servo Max, Rear Link Servo Min, Rear Link Servo Max, Front Link Servo Min, Front Link Servo Max)
  Monster_Crawler.SetServoLimits(0, 90, 180, 0, 90, 180,0, 90, 180);

  // Set the motor configuration
  // (Motor configuration 1 = 1 motor or 2 = 2 motors or 3 = tracks, Do you use a PCF8574 0=no or 1=yes)
  Monster_Crawler.MotorConfiguration(1);

// Set the steering configuration
// (Steering configuration 1 = servo or 2 = motor, Do you use a PCF8574 0=no or 1=yes)
 Monster_Crawler.SteeringConfiguration(2);

  radio.openReadingPipe(1, Controller[0]); // Set Address of Controller 1
  radio.openReadingPipe(2, Controller[1]); // Set Address of Controller 2
  radio.openReadingPipe(3, Controller[2]); // Set Address of Controller 1
  radio.openReadingPipe(4, Controller[3]); // Set Address of Controller 2
  radio.openReadingPipe(5, Controller[4]); // Set Address of Controller 2

  radio.startListening();

    Serial.begin(9600);
    wdt_enable(WDTO_250MS);    
}

void loop(void)
{

    buf[0] = 0;

    if ( radio.available() )
    {
        // Fetch the payload.
       radio.read( buf, sizeof(buf));
       
      // We recieved a string of ten bytes, four ID, one command and six data values but they may not be for this tractor
      if (buf[0] == ID) { // Confirm that the correct vehicle ID has been recieved


      int pipe_num = buf[11];

      long Vehicle_Voltage = Monster_Crawler.ReadVcc();
    
     // First, stop listening so we can talk
      radio.stopListening();

      // Open the correct pipe for writing
      radio.openWritingPipe(Controller_Listen[pipe_num]);

      // Send battery voltage back.
      radio.write( &Vehicle_Voltage, sizeof(Vehicle_Voltage) );

      // Now, resume listening so we catch the next packets.
      radio.startListening();

        int steer_inv = map(buf[3],0,255,255,0);

        Monster_Crawler.Drive(buf[2], steer_inv);
        Monster_Crawler.Lights(buf[9],0);        // Update the lights (Light Data, Invert Output)
        Monster_Crawler.Trailer(buf);	// Send Command to any trailers

        Last_Data_Timer = 10000;
        Active = 1;
      }
    }
    else if (Last_Data_Timer != 0) {
      Last_Data_Timer = Last_Data_Timer - 1;
    }
    else if (Last_Data_Timer == 0 && Active == 1) {
      Active = 0;
      Monster_Crawler.Lights(0x03,0);
      Monster_Crawler.Drive(125, 125);
    }
            
    Monster_Crawler.Timer();
    wdt_reset();           

    /* if (Active == 1) {
      }
      else if (Controller_Locked != 0) {
       All_Controllers();
       Controller_Locked = 0;
      }*/

}